#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem

class TestCreateFS(StorageTestBase):
    """
        Test LMI_FileSystemConfigurationService.LMI_CreateFileSystem
        with different parameters.
    """

    def setUp(self):
        """ Find disk partition service. """
        super(TestCreateFS, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_FileSystemConfigurationService")[0]

    def test_no_params(self):
        """ Try LMI_CreateFileSystem with no parameters"""
        # InExtents = None -> error
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "LMI_CreateFileSystem",
                self.service)

    def test_create_no_fstype(self):
        """ Test LMI_CreateFileSystem with no FileSystemType parameter."""
        # Goal = None -> error
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[0], ])

    def _check_fs(self, fs_name, fs_type, fs_type_name):
        # check the LMI_LocalFileSystem properties
        fs = self.wbemconnection.GetInstance(fs_name)
        self.assertEquals(fs['FileSystemType'], fs_type_name)
        self.assertEquals(fs['PersistenceType'], 2)

        # check the LMI_FileSystemSetting properties
        fs_settings = self.wbemconnection.Associators(fs_name,
                AssocClass="LMI_FileSystemElementSettingData")
        self.assertEquals(len(fs_settings), 1)
        fs_setting = fs_settings[0]
        self.assertEquals(fs_setting['ActualFileSystemType'], fs_type)
        self.assertEquals(fs_setting['ChangeableType'], 3)  # transient
        # TODO: check NumberOfObjects property

    def test_create_ext2(self):
        """ Test LMI_CreateFileSystem with ext2."""
        EXT2 = 11  # LMI_FileSystemSetting.ActualFileSystemType
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[0], ],
                FileSystemType=pywbem.Uint16(EXT2))
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int, "theelement")
        self.assertEquals(ret, 0)
        self.assertIn("theelement", outparams)
        self._check_fs(outparams['theelement'], EXT2, 'ext2')
        self._delete_fs(outparams['theelement'], [self.partition_names[0]])

    def test_create_ext3(self):
        """ Test LMI_CreateFileSystem with ext3."""
        EXT3 = 12  # LMI_FileSystemSetting.ActualFileSystemType
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[0], ],
                FileSystemType=pywbem.Uint16(EXT3))
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int, "theelement")
        self.assertEquals(ret, 0)
        self.assertIn("theelement", outparams)
        self._check_fs(outparams['theelement'], EXT3, 'ext3')
        self._delete_fs(outparams['theelement'], [self.partition_names[0]])

    def test_create_ext4(self):
        """ Test LMI_CreateFileSystem with ext4."""
        EXT4 = 32769  # LMI_FileSystemSetting.ActualFileSystemType
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[0], ],
                FileSystemType=pywbem.Uint16(EXT4))
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int, "theelement")
        self.assertEquals(ret, 0)
        self.assertIn("theelement", outparams)
        self._check_fs(outparams['theelement'], EXT4, 'ext4')
        self._delete_fs(outparams['theelement'], [self.partition_names[0]])

    def test_create_xfs(self):
        """ Test LMI_CreateFileSystem with xfs."""
        XFS = 9  # LMI_FileSystemSetting.ActualFileSystemType
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[0], ],
                FileSystemType=pywbem.Uint16(XFS))
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int, "theelement")
        self.assertEquals(ret, 0)
        self.assertIn("theelement", outparams)
        self._check_fs(outparams['theelement'], XFS, 'xfs')
        self._delete_fs(outparams['theelement'], [self.partition_names[0]])

    def test_create_btrfs(self):
        """ Test LMI_CreateFileSystem with btrfs."""
        BTRFS = 32770  # LMI_FileSystemSetting.ActualFileSystemType
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[0], ],
                FileSystemType=pywbem.Uint16(BTRFS))
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int, "theelement")
        self.assertEquals(ret, 0)
        self.assertIn("theelement", outparams)
        self._check_fs(outparams['theelement'], BTRFS, 'btrfs')
        self._delete_fs(outparams['theelement'], [self.partition_names[0]])

    def test_create_btrfs_2dev(self):
        """ Test LMI_CreateFileSystem with btrfs on 2 devices."""
        BTRFS = 32770  # LMI_FileSystemSetting.ActualFileSystemType
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateFileSystem",
                self.service,
                InExtents=[self.partition_names[1], self.partition_names[2]],
                FileSystemType=pywbem.Uint16(BTRFS))
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int, "theelement")
        self.assertEquals(ret, 0)
        self.assertIn("theelement", outparams)
        self._check_fs(outparams['theelement'], BTRFS, 'btrfs')
        self._delete_fs(outparams['theelement'],
                [self.partition_names[1], self.partition_names[2]])

    def _delete_fs(self, fsname, devicenames):
        """Delete given FS."""
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                "DeleteFileSystem",
                self.service,
                TheFileSystem=fsname)
        if ret == self.JOB_CREATED:
            (ret, outparams) = self.finish_job(
                    outparams['job'], int)
        self.assertEquals(ret, 0)

        for devicename in devicenames:
            filesystems = self.wbemconnection.AssociatorNames(
                    devicename,
                    AssocClass="LMI_ResidesOnExtent")
            self.assertEquals(len(filesystems), 0)



if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
"""
Python Provider for LMI_LUKSFormat

LMI_LUKSFormat
---------------------

.. autoclass:: LMI_LUKSFormat
    :members:

"""

import pywbem
import blivet.formats.luks
from lmi.storage.FormatProvider import FormatProvider
from lmi.storage.util import storage
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_LUKSFormat(FormatProvider):
    """Instrument the CIM class LMI_LUKSFormat

    Class representing LUKS data.

    """

    @cmpi_logging.trace_method
    def __init__ (self, *args, **kwargs):
        super(LMI_LUKSFormat, self).__init__(
            "LMI_LUKSFormat",
            "luks",
            *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_format(self, device, fmt):
        if  isinstance(fmt, blivet.formats.luks.LUKS):
            return True
        return False

    # XXX temporary, remove when blivet can do that
    @cmpi_logging.trace_method
    def _get_luks_key_slot_statuses(self, path):
        import subprocess
        import re
        out = subprocess.check_output(['cryptsetup', 'luksDump', path])
        return map(lambda s: s == 'ENABLED', re.findall('Key Slot \d:\s*(\w+)', out))

    @cmpi_logging.trace_method
    def get_instance(self, env, model, fmt=None):
        model = super(LMI_LUKSFormat, self).get_instance(
                env, model, fmt)
        if not fmt:
            (device, fmt) = self.get_format_for_id(model['Name'])

        model['InstanceID'] = 'luks:' + fmt.device
        if fmt.uuid:
            model['UUID'] = fmt.uuid
        statuses = self._get_luks_key_slot_statuses(fmt.device)
        model['SlotStatus'] = pywbem.CIMProperty(name='SlotStatus',
                                                 value=statuses,
                                                 type='uint16',
                                                 array_size=len(statuses),
                                                 is_array=True)

        return model



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
# -*- coding: utf-8 -*-
"""
Python Provider for LMI_MountedFileSystemCapabilities

Instruments the CIM class LMI_MountedFileSystemCapabilities
"""

import pywbem
import blivet
from lmi.storage.MountingProvider import MountingProvider
from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
from lmi.storage.SettingManager import Setting
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_MountedFileSystemCapabilities(CapabilitiesProvider):
    """
    LMI_MountedFileSystemCapabilities provider implementation.
    """

    @cmpi_logging.trace_method
    def __init__ (self, *args, **kwargs):
        super(LMI_MountedFileSystemCapabilities, self).__init__(
            classname="LMI_MountedFileSystemCapabilities",
            *args, **kwargs)
        self.instances = [
            {
                'InstanceID':'LMI:LMI_MountedFileSystemCapabilities:1',
                'SupportedAsynchronousMethods':[self.Values.SupportedAsynchronousMethods.CreateMount,
                                                self.Values.SupportedAsynchronousMethods.ModifyMount,
                                                self.Values.SupportedAsynchronousMethods.DeleteMount]
                }
            ]

    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
        """
        return self.instances

    # pylint: disable-msg=W0613
    @cmpi_logging.trace_method
    def create_setting_for_capabilities(self, capabilities):
        """
            Create LMI_*Setting for given capabilities.
            Return CIMInstanceName of the setting or raise CIMError on error.
        """
        setting_name = 'LMI_MountedFileSystemSetting'
        setting_id = self.setting_manager.allocate_id(setting_name)
        if not setting_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                  "Failed to allocate setting InstanceID.")

        setting = self.setting_manager.create_setting(setting_name,
                                                      Setting.TYPE_TRANSIENT,
                                                      setting_id)

        setting['ElementName'] = 'CreatedFrom' + capabilities['InstanceID']
        # default settings: rw, suid, dev, exec, auto, nouser, and async.
        setting['AllowWrite'] = True
        setting['AllowSUID'] = True
        setting['InterpretDevices'] = True
        setting['AllowExecution'] = True
        setting['Auto'] = True
        setting['AllowUserMount'] = False
        setting['SynchronousIO'] = False
        self.setting_manager.set_setting(setting_name, setting)

        return pywbem.CIMInstanceName(
                classname=setting_name,
                namespace=self.config.namespace,
                keybindings={'InstanceID': setting_id})

    class Values(object):
        class SupportedAsynchronousMethods(object):
            CreateMount = pywbem.Uint16(0)
            ModifyMount = pywbem.Uint16(1)
            DeleteMount = pywbem.Uint16(2)

        class CreateSetting(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unspecified_Error = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for FormatProvider.

FormatProvider
--------------

.. autoclass:: FormatProvider
    :members:

LMI_ResidesOnExtent
-------------------

.. autoclass:: LMI_ResidesOnExtent
    :members:

"""

import pywbem
from lmi.storage.BaseProvider import BaseProvider
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.util import storage
from lmi.providers.ComputerSystem import get_system_name

class FormatProvider(BaseProvider):
    """
        Abstract provider for data formats and filesystems.
        Each provider must have .device_type property, which represents
        blivet.formats.<DeviceFormat child>.type of format it
        represents.
    """
    @cmpi_logging.trace_method
    def __init__(self, classname, device_type, *args, **kwargs):
        self.classname = classname
        self.device_type = device_type
        super(FormatProvider, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def provides_name(self, name):
        """
            Returns True, if this class is provider for given CIMInstanceName.
            Subclasses do not need to override this method as long as
            self.classname is correct CIM class name.
        """
        if not (name.has_key("CSCreationClassName")
                and name.has_key("CSName")
                and name.has_key("CreationClassName")
                and name.has_key("Name")):
            return False
        if name['CreationClassName'] != self.classname:
            return False
        if name['CSCreationClassName'] != self.config.system_class_name:
            return False
        if name['CSName'] != get_system_name():
            return False
        return True

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def provides_format(self, device, fmt):
        """
            Returns True, if this class is provider for given Anaconda
            DeviceFormat class.
            Subclasses must override this method.
        """
        return False

    @cmpi_logging.trace_method
    def get_uuid(self, device, fmt):
        """
        Return UUID of the format or None when it does not exist.
        """
        try:
            return fmt.uuid
        except AttributeError:
            return None

    @cmpi_logging.trace_method
    def get_format_id(self, device, fmt):
        """
            Return LMI_DataFormat.Name. The name should be unique and stable
            across reboots or reconfigurations. UUID is used, subclasses
            do not need to override this method.
        """
        uuid = self.get_uuid(device, fmt)
        if uuid:
            return "UUID=" + uuid

        return "DEVICE=" + storage.get_persistent_name(device)

    @cmpi_logging.trace_method
    def get_format_for_id(self, name):
        """
            Return tuple (StorageDevice, DeviceFormat) for given Name property
            of CIMInstance. Return (None, None) if no such format exist.
            This is reverse function to get_format_id().
            Subclasses do not need to override this method if they do not
            override get_format_id().
            The returned device represents one randomly chosen device if the
            format spans over multiple devices, use get_devices() to get all
            of them.
        """

        if name.startswith("DEVICE="):
            (_unused, devname) = name.split("=")
            device = storage.get_device_for_persistent_name(self.storage, devname)
        elif name.startswith("UUID="):
            (_unused, uuid) = name.split("=")
            device = self.storage.devicetree.getDeviceByUuid(uuid)
        else:
            return (None, None)
        if not device:
            return (None, None)
        return (device, device.format)

    @cmpi_logging.trace_method
    def get_name_for_format(self, device, fmt):
        """ Return CIMInstanceName for given DeviceFormat subclass."""
        name = pywbem.CIMInstanceName(self.classname,
                namespace=self.config.namespace,
                keybindings={
                        "CSCreationClassName": self.config.system_class_name,
                        "CSName": get_system_name(),
                        "CreationClassName": self.classname,
                        "Name": self.get_format_id(device, fmt)})
        return name

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Enumerate instances. Subclasses do not need to override this method,
            as long as enumeration by self.provides_format is sufficient.
        """
        model.path.update({'CSName': None, 'CreationClassName': None,
            'CSCreationClassName': None, 'Name': None})

        for device in self.storage.devices:
            fmt = device.format
            if fmt and self.provides_format(device, fmt):
                name = self.get_name_for_format(device, fmt)
                model.update(name)
                if keys_only:
                    yield model
                else:
                    yield self.get_instance(env, model, fmt)

    @cmpi_logging.trace_method
    def get_format_for_name(self, instance_name):
        """
            Return tuple (StorageDevice, DeviceFormat) instance for given
            CIMInstanceName. Return (None, None) if no such instance_name
            exists.
            The returned device represents one randomly chosen device if the
            format spans over multiple devices, use get_devices() to get all
            of them.
        """
        if instance_name['CSName'] != get_system_name():
            return (None, None)
        if instance_name['CSCreationClassName'] != \
                    self.config.system_class_name:
            return (None, None)
        if instance_name['CreationClassName'] != self.classname:
            return (None, None)
        return self.get_format_for_id(instance_name['Name'])


    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, fmt=None):
        """
            Get instance.
            Subclasses should override this method, the default implementation
            just checks if the instance exists.
        """
        if not fmt:
            (device, fmt) = self.get_format_for_name(model)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the format.")
        model['FormatTypeDescription'] = fmt.name

        fmt_types = {
                "swap": self.Values.FormatType.Swap,
                "mdmember": self.Values.FormatType.MD_RAID_member,
                "lvmpv": self.Values.FormatType.Physical_Volume,
                "luks": self.Values.FormatType.LUKS,
                "biosboot": self.Values.FormatType.BIOS_Boot,
                "dmraidmember": self.Values.FormatType.DM_RAID_member,
                "multipath_member": self.Values.FormatType.Multipath_member,
                "prepboot": self.Values.FormatType.PPC_PReP_Boot,
        }
        fmt_type = fmt_types.get(fmt.type, self.Values.FormatType.Other)
        model['FormatType'] = fmt_type
        model['ElementName'] = fmt.device
        return model

    def get_devices(self, device, _fmt):
        """
        Return all devices, on which the format resides.
        Usually there is only one device, but btrfs may provide more.
        """
        return [device]


    class Values(object):
        class FormatType(object):
            Swap = pywbem.Uint16(1)
            MD_RAID_member = pywbem.Uint16(2)
            Physical_Volume = pywbem.Uint16(3)
            LUKS = pywbem.Uint16(4)
            BIOS_Boot = pywbem.Uint16(5)
            DM_RAID_member = pywbem.Uint16(6)
            Multipath_member = pywbem.Uint16(7)
            PPC_PReP_Boot = pywbem.Uint16(8)
            Other = pywbem.Uint16(65535)


class LMI_ResidesOnExtent(BaseProvider):
    """ Implementation of LMI_ResidesOnExtent association."""
    def __init__(self, *args, **kwargs):
        super(LMI_ResidesOnExtent, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Enumerate instances.
        """
        model.path.update({'Dependent': None, 'Antecedent': None})

        for device in self.storage.devices:
            fmt = device.format
            if not fmt or not fmt.type:
                continue
            provider = self.provider_manager.get_provider_for_format(
                    device, fmt)
            if not provider:
                continue
            fmtname = provider.get_name_for_format(device, fmt)
            devices = provider.get_devices(device, fmt)
            for device in devices:
                devname = self.provider_manager.get_name_for_device(device)
                if not devname:
                    continue
                model['Dependent'] = fmtname
                model['Antecedent'] = devname
                yield model

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """ Get instance. """
        fmtname = model['Dependent']
        devname = model['Antecedent']

        # just check that the device has the format
        device = self.provider_manager.get_device_for_name(devname)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find Antecedent device.")

        fmtprovider = self.provider_manager.get_provider_for_format_name(
                fmtname)
        if not fmtprovider:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The Antecedent device has unknown format.")

        (device, fmt) = fmtprovider.get_format_for_name(fmtname)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The Antecedent device has unknown format.")

        fmt_devices = fmtprovider.get_devices(device, fmt)
        if device not in fmt_devices:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The Antecedent device has different format.")
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        # don't forget to try both DataFormat and LocalFileSystem subclasses!
        refs = self.simple_references(
                    env, object_name, model,
                    result_class_name, role, result_role, keys_only,
                    "LMI_DataFormat",
                    "CIM_StorageExtent")
        if refs:
            return refs

        refs = self.simple_references(
                    env, object_name, model,
                    result_class_name, role, result_role, keys_only,
                    "LMI_LocalFileSystem",
                    "CIM_StorageExtent")
        return refs



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
# -*- coding: utf-8 -*-
"""
Python Provider for LMI_MountConfigurationService

Instruments the CIM class LMI_MountConfigurationService
"""

import pywbem
import blivet
import subprocess
import os
import re
from lmi.storage.MountingProvider import MountingProvider
from lmi.storage.ServiceProvider import ServiceProvider
from lmi.providers.JobManager import Job
from lmi.storage.util.storage import *
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_MountConfigurationService(ServiceProvider, MountingProvider):
    """Instrument the CIM class LMI_MountConfigurationService

    A Service is a LogicalElement that represents the availability of
    functionality that can be managed. This functionality may be provided
    by a seperately modeled entity such as a LogicalDevice or a
    SoftwareFeature, or both. The modeled Service typically provides only
    functionality required for management of itself or the elements it
    affects.

    """

    _CIM_TO_LINUX_MOUNT_OPTS = {
        'SynchronousIO'               : ('sync', 'async'),
        'SynchronousDirectoryUpdates' : ('dirsync', ''),
        'UpdateAccessTimes'           : ('atime', 'noatime'),
        'UpdateFullAccessTimes'       : ('strictatime', 'nostrictatime'),
        'UpdateRelativeAccessTimes'   : ('relatime', 'norelatime'),
        'UpdateDirectoryAccessTimes'  : ('diratime', 'nodiratime'),
        'InterpretDevices'            : ('dev', 'nodev'),
        'AllowMandatoryLock'          : ('mand', 'nomand'),
        'AllowExecution'              : ('exec', 'noexec'),
        'AllowSUID'                   : ('suid', 'nosuid'),
        'AllowWrite'                  : ('rw', 'ro'),
        'Silent'                      : ('silent', 'loud'),
        'Auto'                        : ('auto', 'noauto'),
        'AllowUserMount'              : ('user', 'nouser'),
        'OtherOptions'                : None
    }

    @cmpi_logging.trace_method
    def __init__ (self, *args, **kwargs):
        super(LMI_MountConfigurationService, self).__init__(
            classname="LMI_MountConfigurationService",
            *args, **kwargs)

    @cmpi_logging.trace_method
    def get_mountopts_from_goal(self, goal):
        if goal:
            mountopts = []
            otheropts = []
            setting = self._parse_goal(goal, 'LMI_MountedFileSystemSetting')

            for k, v in setting.items():
                if v is not None and k in self._CIM_TO_LINUX_MOUNT_OPTS.keys():
                    i = 0 if v == 'True' else 1
                    if k != 'OtherOptions':
                        mountopts.append(self._CIM_TO_LINUX_MOUNT_OPTS[k][i])
                    else:
                        otheropts.extend(re.findall("'(.+?)'", v))
            if otheropts:
                mountopts.extend(otheropts)
        else:
            # use defaults
            mountopts = ['rw', 'suid', 'dev', 'exec', 'auto', 'nouser', 'async']

        return mountopts

    @cmpi_logging.trace_method
    def _parse_goal(self, param_goal, classname):
        """
            Find Setting for given CIMInstanceName and check, that it is
            of given CIM class.
            Return None, if no Goal was given.
            Raise CIMError, if the Goal cannot be found.
        """
        if param_goal:
            instance_id = param_goal['InstanceID']
            goal = self.provider_manager.get_setting_for_id(
                instance_id, classname)
            if not goal:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    classname + " Goal was not found.")
        else:
            goal = None
        return goal

    @cmpi_logging.trace_method
    def cim_method_createmount(self, env, object_name,
                               param_goal=None,
                               param_filesystemtype=None,
                               param_mode=None,
                               param_filesystem=None,
                               param_mountpoint=None,
                               param_filesystemspec=None):
        """Implements LMI_MountConfigurationService.CreateMount()

        Mounts the specified filesystem to a mountpoint.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method CreateMount()
            should be invoked.
        param_goal --  The input parameter Goal (type REF (pywbem.CIMInstanceName(classname='LMI_MountedFileSystemSetting', ...))
            Desired mount settings. If NULL, defaults will be used. Default
            mount options are 'rw, suid, dev, exec, auto, nouser, async'.

        param_filesystemtype --  The input parameter FileSystemType (type unicode)
            Filesystem type. If NULL, perform a binding mount. If mounting
            a local filesystem, this parameter has to be in agreement with
            the FileSystem.

        param_mode --  The input parameter Mode (type pywbem.Uint16 self.Values.CreateMount.Mode)
            The mode in which the configuration is to be applied to the
            MountedFileSystem. IsNext and IsCurrent are properties of
            LMI_MountedFileSystemElementSettingData, which will be
            created. Meaning of IsNext and IsCurrent is:  IsCurrent = 1:
            The filesystem will be mounted. IsCurrent = 2: The filesystem
            will be unmounted. IsNext = 1: A persistent entry will be
            created (in /etc/fstab).  IsNext = 2: The persistent entry
            will be removed.  Mode 1 - IsNext = 1, IsCurrent = 1. Mode 2 -
            IsNext = 1, IsCurrent not affected. Mode 4 - IsNext = 2,
            IsCurrent = 2. Mode 5 - IsNext = 2, IsCurrent not affected.
            Mode 32768 - IsNext not affected, IsCurrent = 1. Mode 32769 -
            IsNext not affected, IsCurrent = 2.

        param_filesystem --  The input parameter FileSystem (type REF (pywbem.CIMInstanceName(classname='CIM_FileSystem', ...))
            Existing filesystem that should be mounted. If NULL, mount a
            remote filesystem, or mount a non-device filesystem (e.g.
            tmpfs). If not NULL, mount a local filesystem. When mounting a
            local filesystem, the FileSystemType parameter has to agree
            with the type of FileSystem.

        param_mountpoint --  The input parameter MountPoint (type unicode)
            Directory where the mounted filesystem should be attached at.

        param_filesystemspec --  The input parameter FileSystemSpec (type unicode)
            Filesystem specification. Specifies the device that should be
            mounted. Remote filesystems can be specified in their usual
            form (e.g. 'hostname:/share' for NFS, or '//hostname/share'
            for CIFS). Non-device filesystems can also be specified (e.g.
            'tmpfs' or 'sysfs'). When performing a bind mount,
            FileSystemSpec is the path to the source directory.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.CreateMount)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Mount -- (type REF (pywbem.CIMInstanceName(classname='LMI_MountedFileSystem', ...))
            Reference to the created LMI_MountedFileSystem instance.

        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the created job.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        # sanity checks
        self.check_instance(object_name)
        args = {'FileSystem':param_filesystem,
                'Mode':param_mode,
                'MountPoint':param_mountpoint,
                'FileSystemSpec':param_filesystemspec}
        check_empty_parameters(**args)
        if param_mode not in [self.MountMethod.Mode.Mode_1,
                              self.MountMethod.Mode.Mode_2,
                              self.MountMethod.Mode.Mode_32768]:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                                  "Mode %d without effect." % param_mode)

        mountopts = self.get_mountopts_from_goal(param_goal)

        if param_filesystem:
            # mount a local filesystem; types and specs have to match
            (device, fmt) = self.get_device_and_format_from_fs(param_filesystem)

            if fmt.type != param_filesystemtype:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                      "Filesystem types don't match.")
            if device.path != param_filesystemspec:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                      "Device paths don't match.")
        else:
            # mount remote/non-device filesystem
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                                  "Can't mount remote/non-device filesystems. Yet...")

        # remember input parameters for job
        input_arguments = {
                'Goal' : pywbem.CIMProperty(name='Goal',
                        type='reference',
                        value=param_goal),
                'FileSystemType' : pywbem.CIMProperty(name='FileSystemType',
                        type='string',
                        value=param_filesystemtype),
                'Mode' : pywbem.CIMProperty(name='Mode',
                        type='uint16',
                        value=param_mode),
                'FileSystem' : pywbem.CIMProperty(name='FileSystem',
                        type='reference',
                        value=param_filesystem),
                'MountPoint' : pywbem.CIMProperty(name='MountPoint',
                        type='string',
                        value=param_mountpoint),
                'FileSystemSpec' : pywbem.CIMProperty(name='FileSystemSpec',
                        type='string',
                        value=param_filesystemspec)
                }

        # create Job
        job = Job(job_manager=self.job_manager,
                  job_name="CREATE MOUNT %s %s (t:%s o:%s)"
                           % (param_filesystemspec,
                              param_mountpoint,
                              param_filesystemtype,
                              ",".join(mountopts)),
                  input_arguments=input_arguments,
                  method_name="CreateMount",
                  affected_elements=[param_filesystem],
                  owning_element=self._get_instance_name())
        job.set_execute_action(self._create_mount, job, param_mode,
                               param_filesystemtype, param_filesystemspec,
                               param_mountpoint, mountopts)
        self.job_manager.add_job(job)

        rval = self.RequestStateChange.Method_Parameters_Checked___Job_Started
        outparams = [(pywbem.CIMParameter(
                name='Job',
                type='reference',
                value=job.get_name()))]
        return (rval, outparams)

    @cmpi_logging.trace_method
    def _create_mount(self, job, mode, fstype, fsspec, mountpoint, mountopts):
        """
        Really create mount. All checks have already been done.
        Mode = 1 --> mount and persistent
        Mode = 2 --> persistent
        Mode = 32768 --> mount
        """
        # mount
        if mode == self.MountMethod.Mode.Mode_1 or \
           mode == self.MountMethod.Mode.Mode_32768:
            rc = self._do_mount(fsspec, mountpoint, fstype, ",".join(mountopts))

            rval = self.MountMethod.Job_Completed_with_No_Error
            state = Job.STATE_FINISHED_OK
            err = None

            if rc != 0:
                rval = self.MountMethod.Failed
                state = Job.STATE_FAILED
                # XXX it will be nice if blivet also returned a nice error
                # message,provide a meaningful error message so it could be
                # provided here
                err = pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                      "Mount failed (%d)." % rc)

        # create fstab entry
        if mode == self.MountMethod.Mode.Mode_1 or \
           mode == self.MountMethod.Mode.Mode_2:
            # XXX not supported yet
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                                  "Persistent entries not supported. Yet...")

        mfsname = pywbem.CIMInstanceName(
            classname='LMI_MountedFileSystem',
            namespace=self.config.namespace,
            keybindings={
                'FileSystemSpec' : fsspec,
                'MountPointPath' : mountpoint
                })
        outparams = {'Mount' : mfsname}
        job.finish_method(
            state,
            return_value=rval,
            return_type=Job.ReturnValueType.Uint32,
            output_arguments=outparams,
            affected_elements=[mfsname],
            error=err)

    @cmpi_logging.trace_method
    def _do_mount(self, device, mountpoint, fstype=None, options=None, bind=False):
        # XXX This is a temporary solution. Blivet's mount() doesn't correctly
        # mimic the system mount command and creates the mountpoint directory if it
        # doesn't exist. We don't want that.
        mountpoint = os.path.normpath(mountpoint)

        argv = ["mount"]
        if bind:
            argv.append("--bind")
        if fstype:
            argv.append("-t")
            argv.append(fstype)
        if device:
            argv.append(device)
        argv.append(mountpoint)
        if options:
            argv.append("-o")
            argv.append(options)

        return subprocess.call(argv)

    @cmpi_logging.trace_method
    def cim_method_deletemount(self, env, object_name,
                               param_mount=None,
                               param_mode=None):
        """Implements LMI_MountConfigurationService.DeleteMount()

        Unmounts an existing mount.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method DeleteMount()
            should be invoked.
        param_mount --  The input parameter Mount (type REF (pywbem.CIMInstanceName(classname='LMI_MountedFileSystem', ...))
            An existing mount.

        param_mode --  The input parameter Mode (type pywbem.Uint16 self.Values.DeleteMount.Mode)
            The mode in which the configuration is to be applied to the
            MountedFileSystem. Mode 1 - IsNext = 1, IsCurrent = 1. Mode 2
            - IsNext = 1, IsCurrent not affected. Mode 4 - IsNext = 2,
            IsCurrent = 2. Mode 5 - IsNext = 2, IsCurrent not affected.
            Mode 32768 - IsNext not affected, IsCurrent = 1. Mode 32769 -
            IsNext not affected, IsCurrent = 2.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.DeleteMount)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the created job.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        # sanity checks
        self.check_instance(object_name)
        args = {'Mount':param_mount,
                'Mode':param_mode}
        check_empty_parameters(**args)
        if param_mode not in [self.MountMethod.Mode.Mode_4,
                              self.MountMethod.Mode.Mode_5,
                              self.MountMethod.Mode.Mode_32769]:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                                  "Mode %d without effect." % param_mode)

        # remember input parameters for job
        input_arguments = {
                'Mount' : pywbem.CIMProperty(name='Mount',
                        type='reference',
                        value=param_mount),
                'Mode' : pywbem.CIMProperty(name='Mode',
                        type='uint16',
                        value=param_mode)
                }

        # create Job
        job = Job(job_manager=self.job_manager,
                  job_name="DELETE MOUNT %s" % param_mount['MountPointPath'],
                  input_arguments=input_arguments,
                  method_name="DeleteMount",
                  affected_elements=[param_mount],
                  owning_element=self._get_instance_name())
        job.set_execute_action(self._delete_mount, job,
                               param_mode, param_mount['MountPointPath'])
        self.job_manager.add_job(job)

        rval = self.RequestStateChange.Method_Parameters_Checked___Job_Started
        outparams = [(pywbem.CIMParameter(
            name='Job',
            type='reference',
            value=job.get_name()))]
        return (rval, outparams)

    @cmpi_logging.trace_method
    def _delete_mount(self, job, mode, mountpoint):
        """
        Really delete mount. All checks have already been done.
        Mode = 4 --> unmount and remove persistent
        Mode = 5 --> remove persistent
        Mode = 32769 --> unmount
        """
        # unmount
        if mode == self.MountMethod.Mode.Mode_4 or \
           mode == self.MountMethod.Mode.Mode_32769:
            rc = blivet.util.umount(mountpoint)

            rval = self.MountMethod.Job_Completed_with_No_Error
            state = Job.STATE_FINISHED_OK
            err = None

            if rc != 0:
                rval = self.MountMethod.Failed
                state = Job.STATE_FAILED
                # XXX it will be nice if blivet also returned a nice error
                # message,provide a meaningful error message so it could be
                # provided here
                err = pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                                      "Unmount failed (%d)." % rc)

        # delete fstab entry
        if mode == self.MountMethod.Mode.Mode_4 or \
           mode == self.MountMethod.Mode.Mode_5:
            # XXX not supported yet
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                                  "Persistent entries not supported. Yet...")

        job.finish_method(
            state,
            return_value=rval,
            return_type=Job.ReturnValueType.Uint32,
            error=err)

    @cmpi_logging.trace_method
    def cim_method_modifymount(self, env, object_name,
                               param_mount=None,
                               param_mode=None,
                               param_goal=None):
        """
        Implements LMI_MountConfigurationService.ModifyMount()

        Modifies (remounts) an existing mount.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method ModifyMount()
            should be invoked.
        param_mount --  The input parameter Mount (type REF (pywbem.CIMInstanceName(classname='LMI_MountedFileSystem', ...))
            Reference to the LMI_Mount instance that is being modified.

        param_mode --  The input parameter Mode (type pywbem.Uint16 self.Values.ModifyMount.Mode)
            The mode in which the configuration is to be applied to the
            MountedFileSystem. Mode 1 - IsNext = 1, IsCurrent = 1. Mode 2
            - IsNext = 1, IsCurrent not affected. Mode 4 - IsNext = 2,
            IsCurrent = 2. Mode 5 - IsNext = 2, IsCurrent not affected.
            Mode 32768 - IsNext not affected, IsCurrent = 1. Mode 32769 -
            IsNext not affected, IsCurrent = 2.

        param_goal --  The input parameter Goal (type REF (pywbem.CIMInstanceName(classname='LMI_MountedFileSystemSetting', ...))
            Desired mount settings. If NULL, the mount options are not
            changed. If mount (or an fstab entry) should be performed
            (created), the appropriate respective MountedFileSystemSetting
            will be created.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.ModifyMount)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the created job.

        Mount -- (type REF (pywbem.CIMInstanceName(classname='LMI_MountedFileSystem', ...))
            Reference to the LMI_Mount instance that is being modified.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # sanity checks
        self.check_instance(object_name)
        args = {'Mount':param_mount,
                'Mode':param_mode}
        check_empty_parameters(**args)
        if param_mode not in [self.MountMethod.Mode.Mode_1,
                              self.MountMethod.Mode.Mode_2,
                              self.MountMethod.Mode.Mode_32768]:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                                  "Mode %d without effect." % param_mode)

        mountopts = self.get_mountopts_from_goal(param_goal)

        # remember input parameters for job
        input_arguments = {
                'Mount' : pywbem.CIMProperty(name='Mount',
                        type='reference',
                        value=param_mount),
                'Mode' : pywbem.CIMProperty(name='Mode',
                        type='uint16',
                        value=param_mode),
                'Goal' : pywbem.CIMProperty(name='Goal',
                        type='reference',
                        value=param_goal)
                }

        mountpoint = param_mount['MountPointPath']
        fsspec = param_mount['FileSystemSpec']

        # create Job
        job = Job(job_manager=self.job_manager,
                  job_name="MODIFY MOUNT %s %s (o:%s)"
                           % (fsspec, mountpoint, ",".join(mountopts)),
                  input_arguments=input_arguments,
                  method_name="ModifyMount",
                  affected_elements=[param_mount],
                  owning_element=self._get_instance_name())
        job.set_execute_action(self._modify_mount, job,
                               fsspec, mountpoint, mountopts)
        self.job_manager.add_job(job)

        rval = self.RequestStateChange.Method_Parameters_Checked___Job_Started
        outparams.append(pywbem.CIMParameter(
                name='Job',
                type='reference',
                value=job.get_name()))
        return (rval, outparams)

    @cmpi_logging.trace_method
    def _modify_mount(self, job, fsspec, mountpoint, mountopts):
        """
        Really modify mount. All checks have been done.
        """
        # XXX just raise error for now
        # there is now way to use blivet to remount stuff now
        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                              "Modifying mounts not supported yet.")

        rval = self.MountMethod.Job_Completed_with_No_Error
        state = Job.STATE_FINISHED_OK
        err = None

        mfsname = pywbem.CIMInstanceName(
            classname='LMI_MountedFileSystem',
            namespace=self.config.namespace,
            keybindings={
                'FileSystemSpec' : fsspec,
                'MountPointPath' : mountpoint
                })
        outparams = {'Mount' : mfsname}

        job.finish_method(
            state,
            return_value=rval,
            return_type=Job.ReturnValueType.Uint32,
            output_arguments=outparams,
            affected_elements=[mfsname],
            error=err)

    class RequestStateChange(object):
        Completed_with_No_Error = pywbem.Uint32(0)
        Not_Supported = pywbem.Uint32(1)
        Unknown_or_Unspecified_Error = pywbem.Uint32(2)
        Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
        Failed = pywbem.Uint32(4)
        Invalid_Parameter = pywbem.Uint32(5)
        In_Use = pywbem.Uint32(6)
        # DMTF_Reserved = ..
        Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
        Invalid_State_Transition = pywbem.Uint32(4097)
        Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
        Busy = pywbem.Uint32(4099)
        # Method_Reserved = 4100..32767
        # Vendor_Specific = 32768..65535

    class MountMethod(object):
        Job_Completed_with_No_Error = pywbem.Uint32(0)
        Not_Supported = pywbem.Uint32(1)
        Unknown = pywbem.Uint32(2)
        Timeout = pywbem.Uint32(3)
        Failed = pywbem.Uint32(4)
        Invalid_Parameter = pywbem.Uint32(5)
        In_Use = pywbem.Uint32(6)
        # DMTF_Reserved = ..
        Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
        # Method_Reserved = 4098..32767
        # Vendor_Specific = 32768..65535
        class Mode(object):
            Mode_0 = pywbem.Uint16(0)
            Mode_1 = pywbem.Uint16(1)
            Mode_2 = pywbem.Uint16(2)
            Mode_3 = pywbem.Uint16(3)
            Mode_4 = pywbem.Uint16(4)
            Mode_5 = pywbem.Uint16(5)
            Mode_6 = pywbem.Uint16(6)
            # DMTF_Reserved = ..
            Mode_32768 = pywbem.Uint16(32768)
            Mode_32769 = pywbem.Uint16(32769)



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for ExtentProvider class.

ExtentProvider
--------------

.. autoclass:: ExtentProvider
    :members:

"""

from lmi.storage.DeviceProvider import DeviceProvider
import pywbem
import blivet.formats
import lmi.storage.util.storage as storage
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.ComputerSystem import get_system_name

class ExtentProvider(DeviceProvider):
    """
        Base of all StorageExtent providers.
        It fills common properties in the StorageExtent.
    """
    @cmpi_logging.trace_method
    def __init__(self, classname, *args, **kwargs):
        self.classname = classname
        super(ExtentProvider, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def _get_device(self, object_name):
        """
            Get Anaconda StorageDevice for given name, without any checks.
        """
        path = object_name['DeviceID']
        device = storage.get_device_for_persistent_name(self.storage, path)
        return device

    @cmpi_logging.trace_method
    def provides_name(self, object_name):
        """
            Returns True, if this class is provider for given CIM InstanceName.
        """
        if not (object_name.has_key('SystemName')
                and object_name.has_key('SystemCreationClassName')
                and object_name.has_key('CreationClassName')
                and object_name.has_key('DeviceID')):
            return False
        if object_name['SystemName'] != get_system_name():
            return False

        if object_name['SystemCreationClassName'] != \
                self.config.system_class_name:
            return False

        if object_name['CreationClassName'] != self.classname:
            return False

        return True

    @cmpi_logging.trace_method
    def get_device_for_name(self, object_name):
        """
            Returns Anaconda StorageDevice for given CIM InstanceName or
            None if no device is found.
        """
        if self.provides_name(object_name):
            return self._get_device(object_name)

    @cmpi_logging.trace_method
    def get_name_for_device(self, device):
        """
            Returns CIM InstanceName for given Anaconda StorageDevice.
            None if no device is found.
        """
        name = pywbem.CIMInstanceName(self.classname,
                namespace=self.config.namespace,
                keybindings={
                    'SystemName' : get_system_name(),
                    'SystemCreationClassName' : self.config.system_class_name,
                    'CreationClassName' : self.classname,
                    'DeviceID': storage.get_persistent_name(device)
                })
        return name

    @cmpi_logging.trace_method
    def get_element_name(self, device):
        """
            Return ElementName property value for given StorageDevice.
            Device path (/dev/sda) is the default.
        """
        return device.name

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_extent_status(self, device):
        """
            Return ExtentStatus property value for given StorageDevice.
            It must be array of int16 values.
        """
        return []

    @cmpi_logging.trace_method
    def get_size(self, device):
        """
            Return (BlockSize, NumberOfBlocks, ConsumableBlocks) properties
            for given StorageDevice.

            The ConsumableBlocks should be reduced by partition table size.
        """
        if device.partedDevice:
            block_size = pywbem.Uint64(device.partedDevice.sectorSize)
            total_blocks = device.partedDevice.length
            consumable_blocks = device.partedDevice.length
            if (device.format and isinstance(device.format,
                    blivet.formats.disklabel.DiskLabel)):
                # reduce by partition table size
                consumable_blocks -= storage.get_partition_table_size(
                        device)
        else:
            block_size = None
            total_blocks = None
            consumable_blocks = None
        return (block_size, total_blocks, consumable_blocks)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_primordial(self, device):
        """
            Returns True, if given StorageDevice is primordial.
        """
        if isinstance(device, blivet.devices.DiskDevice):
            return True
        else:
            return False

    @cmpi_logging.trace_method
    def get_discriminator(self, device):
        """
            Returns ExtentDiscriminator property value for given StorageDevice.
            It must return array of strings.
        """
        discriminator = []
        if device.format and isinstance(device.format,
                    blivet.formats.lvmpv.LVMPhysicalVolume):
            discriminator.append(self.Values.Discriminator.Pool_Component)
        return discriminator

    @cmpi_logging.trace_method
    def get_names(self, device):
        """
        Return array of all aliases (symlinks) of the device. The real
        device path is always the first element of the array.

        :param device: (``StorageDevice``) The device.
        :returns: ``array of strings``.
        """
        return [device.path] + device.deviceLinks

    @cmpi_logging.trace_method
    def get_instance_id(self, device):
        """
        Return InstanceID for given device.

        :param device: (``StorageDevice``) The device.
        :returns: ``string``.
        """
        return ("LMI:" + self.classname + ":"
                + storage.get_persistent_name(device))

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, device=None):
        """
            Provider implementation of GetInstance intrinsic method.
            It fills common StorageExtent properties.
        """
        if not self.provides_name(model):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND, "Wrong keys.")
        if not device:
            device = self._get_device(model)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the extent.")

        model['ElementName'] = self.get_element_name(device)
        model['NameNamespace'] = self.Values.NameNamespace.OS_Device_Namespace
        model['NameFormat'] = self.Values.NameFormat.OS_Device_Name
        model['Name'] = device.path
        model['InstanceID'] = self.get_instance_id(device)

        extent_status = self.get_extent_status(device)
        model['ExtentStatus'] = pywbem.CIMProperty(
                name='ExtentStatus',
                value=extent_status,
                type='uint16',
                array_size=len(extent_status),
                is_array=True)

        operational_status = self.get_status(device)
        model['OperationalStatus'] = pywbem.CIMProperty(
                name='OperationalStatus',
                value=operational_status,
                type='uint16',
                array_size=len(operational_status),
                is_array=True)

        (block_size, total_blocks, consumable_blocks) = self.get_size(device)
        if block_size:
            model['BlockSize'] = pywbem.Uint64(block_size)
        if total_blocks:
            model['NumberOfBlocks'] = pywbem.Uint64(total_blocks)
        if consumable_blocks:
            model['ConsumableBlocks'] = pywbem.Uint64(consumable_blocks)

        redundancy = self.get_redundancy(device)
        model['NoSinglePointOfFailure'] = redundancy.no_single_point_of_failure
        model['DataRedundancy'] = pywbem.Uint16(redundancy.data_redundancy)
        model['PackageRedundancy'] = pywbem.Uint16(
                redundancy.package_redundancy)
        model['ExtentStripeLength'] = pywbem.Uint64(redundancy.stripe_length)
        model['IsComposite'] = (len(device.parents) > 1)

        # TODO: add DeltaReservation (mandatory in SMI-S)

        model['Primordial'] = self.get_primordial(device)
        model['Names'] = self.get_names(device)

        discriminator = self.get_discriminator(device)
        model['ExtentDiscriminator'] = pywbem.CIMProperty(
                name='ExtentDiscriminator',
                value=discriminator,
                type='string',
                array_size=len(discriminator),
                is_array=True)

        return model

    @cmpi_logging.trace_method
    def enumerate_devices(self):
        """
            Enumerate all StorageDevices, that this provider provides.
        """
        pass


    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        model.path.update({'CreationClassName': None, 'SystemName': None,
            'DeviceID': None, 'SystemCreationClassName': None})

        for device in self.enumerate_devices():
            name = self.get_name_for_device(device)
            model['SystemName'] = name['SystemName']
            model['SystemCreationClassName'] = name['SystemCreationClassName']
            model['CreationClassName'] = name['CreationClassName']
            model['DeviceID'] = name['DeviceID']
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, device)

    class Values(DeviceProvider.Values):
        class NameNamespace(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            VPD83Type3 = pywbem.Uint16(2)
            VPD83Type2 = pywbem.Uint16(3)
            VPD83Type1 = pywbem.Uint16(4)
            VPD80 = pywbem.Uint16(5)
            NodeWWN = pywbem.Uint16(6)
            SNVM = pywbem.Uint16(7)
            OS_Device_Namespace = pywbem.Uint16(8)

        class NameFormat(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            VPD83NAA6 = pywbem.Uint16(2)
            VPD83NAA5 = pywbem.Uint16(3)
            VPD83Type2 = pywbem.Uint16(4)
            VPD83Type1 = pywbem.Uint16(5)
            VPD83Type0 = pywbem.Uint16(6)
            SNVM = pywbem.Uint16(7)
            NodeWWN = pywbem.Uint16(8)
            NAA = pywbem.Uint16(9)
            EUI64 = pywbem.Uint16(10)
            T10VID = pywbem.Uint16(11)
            OS_Device_Name = pywbem.Uint16(12)

        class ExtentStatus(object):
            Other = pywbem.Uint16(0)
            Unknown = pywbem.Uint16(1)
            None_Not_Applicable = pywbem.Uint16(2)
            Broken = pywbem.Uint16(3)
            Data_Lost = pywbem.Uint16(4)
            Dynamic_Reconfig = pywbem.Uint16(5)
            Exposed = pywbem.Uint16(6)
            Fractionally_Exposed = pywbem.Uint16(7)
            Partially_Exposed = pywbem.Uint16(8)
            Protection_Disabled = pywbem.Uint16(9)
            Readying = pywbem.Uint16(10)
            Rebuild = pywbem.Uint16(11)
            Recalculate = pywbem.Uint16(12)
            Spare_in_Use = pywbem.Uint16(13)
            Verify_In_Progress = pywbem.Uint16(14)
            In_Band_Access_Granted = pywbem.Uint16(15)
            Imported = pywbem.Uint16(16)
            Exported = pywbem.Uint16(17)
            Relocating = pywbem.Uint16(18)
            # DMTF_Reserved = ..

        class Usage(object):
            Other = pywbem.Uint16(1)
            Unrestricted = pywbem.Uint16(2)
            Reserved_for_ComputerSystem__the_block_server_ = pywbem.Uint16(3)
            Reserved_by_Replication_Services = pywbem.Uint16(4)
            Reserved_by_Migration_Services = pywbem.Uint16(5)
            Local_Replica_Source = pywbem.Uint16(6)
            Remote_Replica_Source = pywbem.Uint16(7)
            Local_Replica_Target = pywbem.Uint16(8)
            Remote_Replica_Target = pywbem.Uint16(9)
            Local_Replica_Source_or_Target = pywbem.Uint16(10)
            Remote_Replica_Source_or_Target = pywbem.Uint16(11)
            Delta_Replica_Target = pywbem.Uint16(12)
            Element_Component = pywbem.Uint16(13)
            Reserved_as_Pool_Contributor = pywbem.Uint16(14)
            Composite_Volume_Member = pywbem.Uint16(15)
            Composite_LogicalDisk_Member = pywbem.Uint16(16)
            Reserved_for_Sparing = pywbem.Uint16(17)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class Discriminator(object):
            Pool_Component = 'SNIA:PoolComponent'
            Composite = 'SNIA:Composite'
            Imported = 'SNIA:Imported'
            Allocated = 'SNIA:Allocated'



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_FileSystemConfigurationService class.

LMI_FileSystemConfigurationService
----------------------------------

.. autoclass:: LMI_FileSystemConfigurationService
    :members:

"""

import blivet.formats
import lmi.storage.util
from lmi.providers.JobManager import Job
from lmi.storage.ServiceProvider import ServiceProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.util import storage

class LMI_FileSystemConfigurationService(ServiceProvider):
    """
        LMI_FileSystemConfigurationService provider implementation.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_FileSystemConfigurationService, self).__init__(
                "LMI_FileSystemConfigurationService", *args, **kwargs)
        self.broker = None

    @cmpi_logging.trace_method
    def cim_method_lmi_createfilesystem(self, env, object_name,
                                        param_elementname=None,
                                        param_goal=None,
                                        param_filesystemtype=None,
                                        param_inextents=None):
        """Implements LMI_FileSystemConfigurationService.LMI_CreateFileSystem()

        Start a job to create a FileSystem on StorageExtents. If the
        operation completes successfully and did not require a
        long-running ConcreteJob, it will return 0. If 4096/0x1000 is
        returned, a ConcreteJob will be started to create the element. A
        Reference to the ConcreteJob will be returned in the output
        parameter Job. If any other value is returned, the job will not be
        started, and no action will be taken. \nThe parameter TheElement
        will contain a Reference to the FileSystem if this operation
        completed successfully. \nThe StorageExtents to use is specified
        by the InExtents parameter.\nThe desired settings for the
        FileSystem are specified by the Goal parameter. Goal is an element
        of class CIM_FileSystemSetting, or a derived class. Unlike CIM
        standard CreateFileSystem, the parameter is reference to
        CIM_FileSystemSetting stored on the CIMOM.\nA ResidesOnExtent
        association is created between the created FileSystem and the
        StorageExtents used for it.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method LMI_CreateFileSystem()
            should be invoked.
        param_elementname --  The input parameter ElementName (type unicode)
            Label of the filesystem being created. If NULL, a
            system-supplied default name can be used. The value will be
            stored in the \'ElementName\' property for the created
            element.

        param_goal --  The input parameter Goal (type REF (pywbem.CIMInstanceName(classname='CIM_FileSystemSetting', ...))
            The requirements for the FileSystem element to maintain. This
            is an element of class CIM_FileSystemSetting, or a derived
            class. This allows the client to specify the properties
            desired for the file system. If NULL, the
            FileSystemConfigurationService will create default filesystem.

        param_filesystemtype --  The input parameter FileSystemType (type pywbem.Uint16 self.Values.LMI_CreateFileSystem.FileSystemType)
            Type of file system to create. When NULL, file system type is
            retrieved from Goal parameter, which cannot be NULL.

        param_inextents --  The input parameter InExtents (type REF (pywbem.CIMInstanceName(classname='CIM_StorageExtent', ...))
            The StorageExtents on which the created FileSystem will reside.
            At least one extent must be provided. If the filesystem being
            created supports more than one storage extent (e.g. btrfs),
            more extents can be provided. The filesystem will then reside
            on all of them.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.LMI_CreateFileSystem)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the job (may be null if job completed).

        TheElement -- (type REF (pywbem.CIMInstanceName(classname='CIM_FileSystem', ...))
            The newly created FileSystem.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        self.check_instance(object_name)

        # remember input parameters for Job
        input_arguments = {
                'ElementName' : pywbem.CIMProperty(name='ElementName',
                        type='string',
                        value=param_elementname),
                'Goal' : pywbem.CIMProperty(name='goal',
                        type='reference',
                        value=param_goal),
                'FileSystemType' : pywbem.CIMProperty(name='FileSystemType',
                        type='uint16',
                        value=param_filesystemtype),
                'InExtents': pywbem.CIMProperty(name='FileSystemType',
                        type='reference',
                        is_array=True,
                        value=param_inextents),
        }

        if not param_inextents:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter InExtents must be specified.")
        devices = []
        for extent in param_inextents:
            device = self.provider_manager.get_device_for_name(extent)
            if not device:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Cannot find block device for InExtent"
                    + extent['DeviceID'])
            devices.append(device)
        if len(devices) > 1:
            if (param_filesystemtype
                    != self.Values.LMI_CreateFileSystem.FileSystemType.BTRFS):
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Selected filesystem supports only one device.")
        if len(devices) < 1:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "At least one InExtent must be specified")
        # Check the devices are unused
        storage.assert_unused(self.storage, devices)

        # Convert devices to strings, so we can survive if some of them
        # disappears or is changed while the job is queued.
        device_strings = [device.path for device in devices]

        goal = self._parse_goal(param_goal, "LMI_FileSystemSetting")
        # TODO: check that goal has supported values

        if not goal and not param_filesystemtype:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Either Goal or FileSystemType parameter must be specified")
        if not param_filesystemtype:
            # retrieve fs type from the goal
            param_filesystemtype = int(goal['ActualFileSystemType'])

        # convert fstype to Blivet FS
        types = self.Values.LMI_CreateFileSystem.FileSystemType
        fstypes = {
                types.FAT: 'vfat',
                types.FAT16: 'vfat',
                types.FAT32: 'vfat',
                types.XFS: 'xfs',
                types.EXT2: 'ext2',
                types.EXT3: 'ext3',
                types.EXT4: 'ext4',
                types.BTRFS: 'btrfs',
                types.VFAT: 'vfat'
        }
        fsname = fstypes.get(param_filesystemtype, 'None')
        if not fsname:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Creation of requested filesystem is not supported.")

        # create the format
        fmt = blivet.formats.getFormat(fsname)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Creation of requested filesystem is not supported.")

        # prepare job
        job = Job(
                job_manager=self.job_manager,
                job_name="CREATE FS " + fsname + " ON " + device.path,
                input_arguments=input_arguments,
                method_name='LMI_CreateFileSystem',
                affected_elements=param_inextents,
                owning_element=self._get_instance_name())
        job.set_execute_action(self._create_fs,
                job, device_strings, fmt, param_elementname, goal)

        # prepare output arguments
        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        retvals = self.Values.LMI_CreateFileSystem

        # enqueue the job
        self.job_manager.add_job(job)
        return (retvals.Method_Parameters_Checked___Job_Started,
                outparams)

    @cmpi_logging.trace_method
    def _parse_goal(self, param_goal, classname):
        """
            Find Setting for given CIMInstanceName and check, that it is
            of given CIM class.
            Return None, if no Goal was given.
            Raise CIMError, if the Goal cannot be found.
        """
        if param_goal:
            instance_id = param_goal['InstanceID']
            goal = self.provider_manager.get_setting_for_id(
                instance_id, classname)
            if not goal:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    classname + " Goal does not found.")
            goal.touch()
        else:
            goal = None
        return goal

    @cmpi_logging.trace_method
    def _create_fs(self, job, device_strings, fmt, label, goal):
        """
            Create a filesystem on given devices. This method is called
            from JobManager worker thread!
        """
        devices = []
        # convert strings back to devices
        for devname in device_strings:
            device = self.storage.devicetree.getDeviceByPath(devname)
            if not device:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "One of the devices disappeared: " + devname)
            devices.append(device)
        # Check the devices are unused
        storage.assert_unused(self.storage, devices)

        if fmt.type == 'btrfs':
            # BTRFS is different beast, we must create BTRFSVolumeDevice
            for device in devices:
                device.format = blivet.formats.getFormat('btrfs')
            volume = self.storage.newBTRFS(
                    fmt_args={'label': label},
                    parents=devices)
            action = blivet.ActionCreateDevice(volume)
        else:
            # create simple CreateFormat action
            action = blivet.ActionCreateFormat(devices[0],
                    format=fmt)

        lmi.storage.util.storage.do_storage_action(
                self.storage, [action])

        # We must locate the format manually, the storage was reset
        # TODO: remove when reset() is not necessary
        device = self.storage.devicetree.getDeviceByPath(devices[0].path)
        if device:
            fmt = device.format
        else:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot locate new format, was it removed?")

        fmtprovider = self.provider_manager.get_provider_for_format(
                device, fmt)
        format_name = fmtprovider.get_name_for_format(device, fmt)
        outparams = {
            'theelement': format_name
        }

        ret = self.Values.LMI_CreateFileSystem.Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[format_name, ],
                error=None)

    @cmpi_logging.trace_method
    def cim_method_deletefilesystem(self, env, object_name,
                                    param_waittime=None,
                                    param_thefilesystem=None,
                                    param_inuseoptions=None):
        """Implements LMI_FileSystemConfigurationService.DeleteFileSystem()

        Start a job to delete a FileSystem. If the FileSystem cannot be
        deleted, no action will be taken, and the Return Value will be
        4097/0x1001. If the method completed successfully and did not
        require a long-running ConcreteJob, it will return 0. If
        4096/0x1000 is returned, a ConcreteJob will be started to delete
        the FileSystem. A Reference to the ConcreteJob will be returned in
        the output parameter Job.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method DeleteFileSystem()
            should be invoked.
        param_waittime --  The input parameter WaitTime (type pywbem.Uint32)
            An integer that indicates the time (in seconds) that the
            provider must wait before deleting this FileSystem. If
            WaitTime is not zero, the method will create a job, if
            supported by the provider, and return immediately. If the
            provider does not support asynchronous jobs, there is a
            possibility that the client could time-out before the job is
            completed.  The combination of InUseOptions = '4' and WaitTime
            ='0' (the default) is interpreted as 'Wait (forever) until
            Quiescence, then Delete Filesystem' and will be performed
            asynchronously if possible.

        param_thefilesystem --  The input parameter TheFileSystem (type REF (pywbem.CIMInstanceName(classname='CIM_ManagedElement', ...))
            An element or association that uniquely identifies the
            FileSystem to be deleted.

        param_inuseoptions --  The input parameter InUseOptions (type pywbem.Uint16 self.Values.DeleteFileSystem.InUseOptions)
            An enumerated integer that specifies the action to take if the
            FileSystem is still in use when this request is made.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.DeleteFileSystem)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the job (may be null if job completed).


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        self.check_instance(object_name)

        # remember input parameters for Job
        input_arguments = {
                'WaitTime' : pywbem.CIMProperty(name='WaitTime',
                        type='uint32',
                        value=param_waittime),
                'TheFileSystem' : pywbem.CIMProperty(name='TheFileSystem',
                        type='reference',
                        value=param_thefilesystem),
                'InUseOptions' : pywbem.CIMProperty(name='InUseOptions',
                        type='uint16',
                        value=param_inuseoptions),
        }

        if param_waittime is not None:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Parameter WaitTime is not supported.")
        if param_inuseoptions is not None:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Parameter InUseOptions is not supported.")
        if param_thefilesystem is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Parameter TheFileSystem must be specified.")
        provider = self.provider_manager.get_provider_for_format_name(
                param_thefilesystem)
        if not provider:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown TheFileSystem class.")
        (device, fmt) = provider.get_format_for_name(param_thefilesystem)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown TheFileSystem instance.")
        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        # prepare job
        job = Job(
                job_manager=self.job_manager,
                job_name="DELETE FS " + fmt.type + " ON " + fmt.device,
                input_arguments=input_arguments,
                method_name='DeleteFileSystem',
                affected_elements=param_thefilesystem,
                owning_element=self._get_instance_name())
        job.set_execute_action(self._delete_fs,
                job, param_thefilesystem)

        # prepare output arguments
        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        retvals = self.Values.LMI_CreateFileSystem

        # enqueue the job
        self.job_manager.add_job(job)
        return (retvals.Method_Parameters_Checked___Job_Started,
                outparams)

    @cmpi_logging.trace_method
    def _delete_fs(self, job, param_thefilesystem):
        """
            Delete a filesystem This method is called from JobManager worker
            thread.
        """
        provider = self.provider_manager.get_provider_for_format_name(
                param_thefilesystem)
        if not provider:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown TheFileSystem class.")
        (device, fmt) = provider.get_format_for_name(param_thefilesystem)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown TheFileSystem instance, the filesystem is "\
                    " probably already deleted.")
        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        actions = []
        if isinstance(device, blivet.devices.BTRFSVolumeDevice):
            # delete the volume device first
            actions.append(blivet.ActionDestroyDevice(device))
            for parent in device.parents:
                # and destroy all formats
                actions.append(blivet.ActionDestroyFormat(parent))
        else:
            actions.append(blivet.ActionDestroyFormat(device))
        lmi.storage.util.storage.do_storage_action(
                self.storage, actions)

        ret = self.Values.DeleteFileSystem.Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=[],
                affected_elements=[],
                error=None)

    class Values(ServiceProvider.Values):
        class LMI_CreateFileSystem(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            StorageExtent_is_not_big_enough_to_satisfy_the_request_ = \
                    pywbem.Uint32(6)
            StorageExtent_specified_by_default_cannot_be_created_ = \
                    pywbem.Uint32(7)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535
            class FileSystemType(object):
                Unknown = pywbem.Uint16(0)
                UFS = pywbem.Uint16(2)
                HFS = pywbem.Uint16(3)
                FAT = pywbem.Uint16(4)
                FAT16 = pywbem.Uint16(5)
                FAT32 = pywbem.Uint16(6)
                NTFS4 = pywbem.Uint16(7)
                NTFS5 = pywbem.Uint16(8)
                XFS = pywbem.Uint16(9)
                AFS = pywbem.Uint16(10)
                EXT2 = pywbem.Uint16(11)
                EXT3 = pywbem.Uint16(12)
                REISERFS = pywbem.Uint16(13)
                # DMTF_Reserved = ..
                EXT4 = pywbem.Uint16(32769)
                BTRFS = pywbem.Uint16(32770)
                JFS = pywbem.Uint16(32771)
                TMPFS = pywbem.Uint16(32772)
                VFAT = pywbem.Uint16(32773)
        class DeleteFileSystem(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed__Unspecified_Reasons = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            FileSystem_in_use__Failed = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            # Method_Parameters_Checked___Job_Started = 0x1000
            # Method_Reserved = 0x1001..0x7FFF
            # Vendor_Specific = 0x8000..
            class InUseOptions(object):
                Do_Not_Delete = pywbem.Uint16(2)
                Wait_for_specified_time__then_Delete_Immediately = \
                    pywbem.Uint16(3)
                Attempt_Quiescence_for_specified_time__then_Delete_Immediately \
 = pywbem.Uint16(4)
                # DMTF_Reserved = ..
                # Vendor_Defined = 0x1000..0xFFFF



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_VGStorageCapabilities class.

LMI_VGStorageCapabilities
-------------------------

.. autoclass:: LMI_VGStorageCapabilities
    :members:

"""

from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
from lmi.storage.SettingManager import Setting
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging
import lmi.storage.util.units as units
from lmi.storage.DeviceProvider import DeviceProvider

DEFAULT_EXTENT_SIZE = 4 * units.MEGABYTE

class LMI_VGStorageCapabilities(CapabilitiesProvider):
    """
        LMI_VGStorageCapabilities provider implementation.
    """

    INSTANCE_ID = "LMI:LMI_VGStorageCapabilities:Global"

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_VGStorageCapabilities, self).__init__(
                "LMI_VGStorageCapabilities", *args, **kwargs)

        self.instances = [
            {
                    'InstanceID': LMI_VGStorageCapabilities.INSTANCE_ID,
                    'ElementName': LMI_VGStorageCapabilities.INSTANCE_ID,
                    'DataRedundancyDefault': pywbem.Uint16(1),
                    'DataRedundancyMax': pywbem.Uint16(units.MAXINT16),
                    'DataRedundancyMin': pywbem.Uint16(1),
                    'NoSinglePointOfFailure': True,
                    'NoSinglePointOfFailureDefault': False,
                    'ExtentStripeLengthDefault': pywbem.Uint16(1),
                    'PackageRedundancyDefault': pywbem.Uint16(0),
                    'PackageRedundancyMax': pywbem.Uint16(units.MAXINT16),
                    'PackageRedundancyMin': pywbem.Uint16(0),
                    'ExtentSizeDefault': pywbem.Uint64(DEFAULT_EXTENT_SIZE),
                    'ParityLayoutDefault':  pywbem.Uint16(0)
            },
    ]

    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
        """

        return self.instances

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def create_setting_for_capabilities(self, capabilities, default=False):
        """
            Create LMI_VGStorageSetting for given capabilities.
            Return CIMInstanceName of the setting or raise CIMError on error.
        """
        setting_id = self.setting_manager.allocate_id(
                'LMI_VGStorageSetting')
        if not setting_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Failed to allocate setting InstanceID")

        setting = self.setting_manager.create_setting(
                'LMI_VGStorageSetting',
                Setting.TYPE_TRANSIENT,
                setting_id)

        if default:
            setting['ExtentSize'] = pywbem.Uint64(
                    capabilities['ExtentSizeDefault'])
            setting['DataRedundancyGoal'] = pywbem.Uint16(
                    capabilities['DataRedundancyDefault'])
            setting['DataRedundancyMax'] = pywbem.Uint16(
                    capabilities['DataRedundancyDefault'])
            setting['DataRedundancyMin'] = pywbem.Uint16(
                    capabilities['DataRedundancyDefault'])
            setting['ExtentStripeLength'] = pywbem.Uint16(
                    capabilities['ExtentStripeLengthDefault'])
            setting['ExtentStripeLengthMax'] = pywbem.Uint16(
                    capabilities['ExtentStripeLengthDefault'])
            setting['ExtentStripeLengthMin'] = pywbem.Uint16(
                    capabilities['ExtentStripeLengthDefault'])
            setting['NoSinglePointOfFailure'] = \
                    capabilities['NoSinglePointOfFailureDefault']
            setting['PackageRedundancyGoal'] = pywbem.Uint16(
                    capabilities['PackageRedundancyDefault'])
            setting['PackageRedundancyMax'] = pywbem.Uint16(
                    capabilities['PackageRedundancyDefault'])
            setting['PackageRedundancyMin'] = pywbem.Uint16(
                    capabilities['PackageRedundancyDefault'])
            if capabilities['ParityLayoutDefault']:
                setting['ParityLayout'] = pywbem.Uint16(
                        capabilities['ParityLayoutDefault'] - 1)
            else:
                setting['ParityLayout'] = None
        else:
            setting['ExtentSize'] = pywbem.Uint64(
                    capabilities['ExtentSizeDefault'])
            setting['DataRedundancyGoal'] = pywbem.Uint16(
                    capabilities['DataRedundancyDefault'])
            setting['DataRedundancyMax'] = pywbem.Uint16(
                    capabilities['DataRedundancyMax'])
            setting['DataRedundancyMin'] = pywbem.Uint16(
                    capabilities['DataRedundancyMin'])
            setting['ExtentStripeLength'] = pywbem.Uint16(
                    capabilities['ExtentStripeLengthDefault'])
            setting['ExtentStripeLengthMax'] = pywbem.Uint16(
                    capabilities['ExtentStripeLengthDefault'])
            setting['ExtentStripeLengthMin'] = pywbem.Uint16(
                    capabilities['ExtentStripeLengthDefault'])
            setting['NoSinglePointOfFailure'] = \
                    capabilities['NoSinglePointOfFailureDefault']
            setting['PackageRedundancyGoal'] = pywbem.Uint16(
                    capabilities['PackageRedundancyDefault'])
            setting['PackageRedundancyMax'] = pywbem.Uint16(
                    capabilities['PackageRedundancyMax'])
            setting['PackageRedundancyMin'] = pywbem.Uint16(
                    capabilities['PackageRedundancyMin'])
            if capabilities['ParityLayoutDefault']:
                setting['ParityLayout'] = pywbem.Uint16(
                        capabilities['ParityLayoutDefault'] - 1)
            else:
                setting['ParityLayout'] = None

        setting['ElementName'] = 'CreatedFrom' + capabilities['InstanceID']
        self.setting_manager.set_setting('LMI_VGStorageSetting', setting)
        return pywbem.CIMInstanceName(
                classname='LMI_VGStorageSetting',
                namespace=self.config.namespace,
                keybindings={'InstanceID': setting_id})


    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def cim_method_createsetting(self, env, object_name,
                                 param_settingtype=None):
        """
            Implements LMI_VGStorageCapabilities.CreateSetting()

            Method to create and populate a StorageSetting instance from a
            StorageCapability instance. This removes the need to populate
            default settings and other settings in the context of each
            StorageCapabilities (which could be numerous). If the underlying
            instrumentation supports the StorageSettingWithHints subclass,
            then an instance of that class will be created instead.
        """
        default = False
        if param_settingtype:
            setting_types = self.Values.CreateSetting.SettingType
            if param_settingtype == setting_types.Default:
                default = True
            elif param_settingtype == setting_types.Goal:
                default = False
            else:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Parameter SettingType has invalid value.")

        capabilities = self.get_capabilities_for_id(object_name['InstanceID'])
        if not capabilities:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the Capabilities.")

        setting_name = self.create_setting_for_capabilities(
                capabilities, default)

        outparams = [pywbem.CIMParameter('newsetting', type='reference',
                           value=setting_name)]
        rval = self.Values.CreateSetting.Success
        return (rval, outparams)

    @cmpi_logging.trace_method
    def cim_method_createvgstoragesetting(self, env, object_name,
                                          param_inextents=None):
        """
            Implements LMI_VGStorageCapabilities.CreateVGStorageSetting()

            This method creates new instance of LMI_VGStorageSetting.
            Applications then do not need to calculate DataRedundancy,
            PackageRedundancy and ExtentStripeLength.
        """
        if not param_inextents:
            # just return default setting
            return self.cim_method_createsetting(env, object_name)

        # find the common redundancy

        # create list of redundancies of all input devices
        # (assuming 'linear' composition)
        redundancies = []
        for device_name in param_inextents:
            provider = self.provider_manager.get_device_provider_for_name(
                    device_name)
            if not provider:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        "Cannot find provider for one of the input extents.")
            device = provider.get_device_for_name(device_name)
            if not device:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        "Cannot find device for one of the input extents.")
            redundancy = provider.get_redundancy(device)
            redundancies.append(redundancy)

        final_redundancy = DeviceProvider.Redundancy.get_common_redundancy_list(
                redundancies)

        setting_id = self.setting_manager.allocate_id(
                'LMI_VGStorageSetting')
        if not setting_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Failed to allocate setting InstanceID")

        setting = self.setting_manager.create_setting(
                'LMI_VGStorageSetting',
                Setting.TYPE_TRANSIENT,
                setting_id)
        setting['ExtentSize'] = pywbem.Uint64(DEFAULT_EXTENT_SIZE)
        setting['DataRedundancyGoal'] = pywbem.Uint16(
                final_redundancy.data_redundancy)
        setting['DataRedundancyMax'] = pywbem.Uint16(
                final_redundancy.data_redundancy)
        setting['DataRedundancyMin'] = pywbem.Uint16(
                final_redundancy.data_redundancy)
        setting['ExtentStripeLength'] = pywbem.Uint16(
                final_redundancy.stripe_length)
        setting['ExtentStripeLengthMax'] = pywbem.Uint16(
                final_redundancy.stripe_length)
        setting['ExtentStripeLengthMin'] = pywbem.Uint16(
                final_redundancy.stripe_length)
        setting['NoSinglePointOfFailure'] = \
                final_redundancy.no_single_point_of_failure
        setting['PackageRedundancyGoal'] = pywbem.Uint16(
                final_redundancy.package_redundancy)
        setting['PackageRedundancyMax'] = pywbem.Uint16(
                final_redundancy.package_redundancy)
        setting['PackageRedundancyMin'] = pywbem.Uint16(
                final_redundancy.package_redundancy)
        if final_redundancy.parity_layout is not None:
            setting['ParityLayout'] = pywbem.Uint16(
                    final_redundancy.parity_layout)
        else:
            setting['ParityLayout'] = None

        setting['ElementName'] = 'CreatedFrom' + object_name['InstanceID']
        self.setting_manager.set_setting('LMI_VGStorageSetting', setting)

        outparams = [ pywbem.CIMParameter('setting', type='reference',
                           value=pywbem.CIMInstanceName(
                                   classname='LMI_VGStorageSetting',
                                   namespace=self.config.namespace,
                                   keybindings={'InstanceID': setting_id}))]
        return (self.Values.CreateVGStorageSetting.Success, outparams)

    class Values(CapabilitiesProvider.Values):
        class CreateSetting(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unspecified_Error = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535
            class SettingType(object):
                Default = pywbem.Uint16(2)
                Goal = pywbem.Uint16(3)

        class CreateVGStorageSetting(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Failed = pywbem.Uint32(4)

        class ParityLayoutDefault(object):
            Non_Rotated_Parity = pywbem.Uint16(2)
            Rotated_Parity = pywbem.Uint16(3)



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-

from lmi.storage.StorageConfiguration import StorageConfiguration
import lmi.providers.cmpi_logging as cmpi_logging
import unittest

import logging
import socket
import os

class TestConfig(unittest.TestCase):
    def setUp(self):
        self.directory = os.path.dirname(__file__)
        if not self.directory:
            self.directory = "."
        cmpi_logging.logger = logging.getLogger('lmi.storage')

    def test_missing(self):
        """ Test configuration when CONFIG_FILE cannot be found."""

        StorageConfiguration.CONFIG_PATH = self.directory
        StorageConfiguration.CONFIG_FILE = self.directory + "/configs/not-existing.conf"
        cfg = StorageConfiguration()

        # cfg must exists
        self.assertTrue(cfg)

        # cfg must return default values
        self.assertEqual(cfg.namespace, "root/cimv2")
        self.assertEqual(cfg.system_class_name, "PG_ComputerSystem")
        self.assertEqual(cfg.system_name, socket.getfqdn())

    def test_empty(self):
        """ Test configuration when CONFIG_FILE is empty."""
        StorageConfiguration.CONFIG_PATH = self.directory
        StorageConfiguration.CONFIG_FILE = self.directory + "/configs/empty.conf"
        cfg = StorageConfiguration()

        # cfg must exists
        self.assertTrue(cfg)

        # cfg must return default values
        self.assertEqual(cfg.namespace, "root/cimv2")
        self.assertEqual(cfg.system_class_name, "PG_ComputerSystem")
        self.assertEqual(cfg.system_name, socket.getfqdn())

    def test_full(self):
        """ Test configuration when CONFIG_FILE is complete."""
        StorageConfiguration.CONFIG_PATH = self.directory
        StorageConfiguration.CONFIG_FILE = self.directory + "/configs/complete.conf"
        cfg = StorageConfiguration()

        # cfg must exists
        self.assertTrue(cfg)

        # cfg must NOT return default values
        self.assertEqual(cfg.namespace, "root/my/namespace")
        self.assertEqual(cfg.system_class_name, "My_ComputerSystem")
        self.assertEqual(cfg.system_name, socket.getfqdn())

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_DiskPartition class.

LMI_DiskPartition
-----------------

.. autoclass:: LMI_DiskPartition
    :members:

"""

from lmi.storage.ExtentProvider import ExtentProvider
import blivet
import pywbem
import lmi.storage.util.storage as storage
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_DiskPartition(ExtentProvider):
    """
        Provider of LMI_DiskPartition class.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_DiskPartition, self).__init__(
                'LMI_DiskPartition', *args, **kwargs)


    @cmpi_logging.trace_method

    def provides_device(self, device):
        """
            Returns True, if this class is provider for given Anaconda
            StorageDevice class.
        """
        if  isinstance(device, blivet.devices.PartitionDevice):
            if device.disk.format.labelType == 'msdos':
                return True
        return False

    @cmpi_logging.trace_method
    def get_base_devices(self, device):
        if device.isPrimary or device.isExtended:
            return super(LMI_DiskPartition, self).get_base_devices(device)

        # logical partitions depend on the extended partition
        parted_ext = device.disk.format.partedDisk.getExtendedPartition()
        if not parted_ext:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    'Cannot find extended partition for device: ' + device.path)

        ext = self.storage.devicetree.getDeviceByPath(parted_ext.path)
        return [ext, ]


    @cmpi_logging.trace_method

    def enumerate_devices(self):
        """
            Enumerate all StorageDevices, that this provider provides.
        """
        for device in self.storage.partitions:
            if self.provides_device(device):
                yield device

    @cmpi_logging.trace_method
    def get_instance(self, env, model, device=None):
        """
            Add partition-specific properties.
        """
        model = super(LMI_DiskPartition, self).get_instance(env, model, device)
        if not device:
            device = self._get_device(model)

        model['PrimaryPartition'] = device.isPrimary
        if device.isPrimary:
            model['PartitionType'] = self.Values.PartitionType.Primary
        if device.isExtended:
            model['PartitionType'] = self.Values.PartitionType.Extended
        if device.isLogical:
            model['PartitionType'] = self.Values.PartitionType.Logical

        return model

    @cmpi_logging.trace_method
    def do_delete_instance(self, device):
        """
            Really delete given Anaconda StorageDevice.
        """
        storage.log_storage_call("DELETE PARTITION",
                {'device': device})
        storage.remove_partition(self.storage, device)

    class Values(ExtentProvider.Values):
        class PartitionType(object):
            Unknown = pywbem.Uint16(0)
            Primary = pywbem.Uint16(1)
            Extended = pywbem.Uint16(2)
            Logical = pywbem.Uint16(3)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>


from test_base import StorageTestBase
import unittest
import pywbem
import subprocess
import time
import re

class TestMounting(StorageTestBase):
    @classmethod
    def setUpClass(cls):
        super(TestMounting, cls).setUpClass()
        cls.mnt_partitions = {}
        i = 0
        for fstype in ["ext2", "ext3", "ext4", "btrfs", "xfs"]:
            cls.mnt_partitions[fstype] = cls.partitions[i]
            if fstype in ["btrfs", "xfs"]:
                subprocess.check_call(["mkfs." + fstype, "-f", cls.partitions[i]])
            else:
                subprocess.check_call(["mkfs." + fstype, cls.partitions[i]])
            i += 1
        TestMounting._restart_cim()

    @classmethod
    def tearDownClass(cls):
        for partition in cls.mnt_partitions.itervalues():
            subprocess.check_call(["wipefs", "-a", partition])
        # other tests might run afterwards, so restart cimom to reset blivet
        TestMounting._restart_cim()

    @classmethod
    def _restart_cim(cls):
        subprocess.check_call(["systemctl", "restart", cls.cimom])
        time.sleep(1)

    def setUp(self):
        super(TestMounting, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames("LMI_MountConfigurationService")[0]
        self.capability = self.wbemconnection.EnumerateInstanceNames("LMI_MountedFileSystemCapabilities")[0]

    def tearDown(self):
        super(TestMounting, self).tearDown()

    def _create_setting(self):
        (ret, outparams) = self.wbemconnection.InvokeMethod('CreateSetting', self.capability)
        self.assertEqual(ret, 0)
        return outparams['setting']

    def _mount(self, mnt_point, partition, fstype, setting_name=None):
        assoc_elems = self.wbemconnection.ExecQuery("WQL",
                                                    'select * from LMI_FileSystemSetting where \
                                                    InstanceID="LMI:LMI_FileSystemSetting:%s"' % (partition))
        if not assoc_elems:
            self.fail("No associations with '%s'" % (partition))

        fs_name = self.wbemconnection.AssociatorNames(assoc_elems[0].path)[0]

        kwargs = {"FileSystemType":fstype,
                  "Mode"          :pywbem.Uint16(32768),
                  "FileSystem"    :fs_name,
                  "MountPoint"    :mnt_point,
                  "FileSystemSpec":partition}
        if setting_name is not None:
            kwargs["Goal"] = setting_name

        (ret, outparams) = self.invoke_async_method(
            "CreateMount",
            self.service,
            int,
            **kwargs)

        # 0 - Completed with no error
        # 4096 - Method parameters checked, job started
        self.assertIn(ret, (0, 4096))
        # XXX outparams is always {} here, even though mounting was successful. BUG?

    def _modify_mount(self, something):
        # TBI
        pass

    def _umount(self, mnt_point, partition, fstype='ext4'):
        mnt_name = pywbem.CIMInstanceName(classname='LMI_MountedFileSystem',
                                          namespace='root/cimv2',
                                          keybindings={'MountPointPath':mnt_point,
                                                       'FileSystemType':fstype,
                                                       'FileSystemSpec':partition})

        (ret, outparams) = self.invoke_async_method("DeleteMount",
                                                    self.service,
                                                    int,
                                                    Mount=mnt_name,
                                                    Mode=pywbem.Uint16(32769)
                                                    )

        self.assertEqual(ret, 0)
        self.assertEqual(outparams, {})

    def _grep_mount(self, partition):
        p1 = subprocess.Popen(["mount"], stdout=subprocess.PIPE)
        p2 = subprocess.Popen(["grep", partition], stdin=p1.stdout, stdout=subprocess.PIPE)
        p1.stdout.close()
        return p2.communicate()[0]

    def _check_partition_is_mounted(self, partition):
        out = self._grep_mount(partition)
        self.assertRegexpMatches(out, partition + " on " + self.mnt_dir)

    def _check_partition_is_unmounted(self, partition):
        out = self._grep_mount(partition)
        self.assertEquals(out, "")

    def _create_id(self, spec, path):
        return spec + '|' + path

    def test_simple_mount_and_umount(self):
        for fstype, partition in self.mnt_partitions.iteritems():
            # mount
            try:
                self._mount(self.mnt_dir, partition, fstype)
            except pywbem.CIMError as pe:
                self.fail(pe[1])
            out = self._grep_mount(partition)
            self._check_partition_is_mounted(partition)

            # check the default setting
            sid = partition + "|" + self.mnt_dir
            assoc_elems = self.wbemconnection.ExecQuery("WQL",
                                                        'select * from LMI_MountedFileSystemSetting where \
                                                        InstanceID="LMI:LMI_MountedFileSystemSetting:%s"' % (sid))
            if not assoc_elems:
                self.fail("No associations with '%s'" % (partition))
            self.assertTrue(assoc_elems[0]['AllowWrite'])
            self.assertTrue(assoc_elems[0]['UpdateRelativeAccessTimes'])
            if fstype != 'btrfs':
                # Blivet just says 'defaults', so look for that
                self.assertEquals(assoc_elems[0]['OtherOptions'], ['defaults'])

            # unmount
            try:
                self._umount(self.mnt_dir, partition, fstype)
            except pywbem.CIMError as pe:
                self.fail(pe[1])
            out = self._grep_mount(partition)
            self._check_partition_is_unmounted(partition)

    def test_mount_and_umount_wrong_dir(self):
        for fstype, partition in self.mnt_partitions.iteritems():
            self.assertRaises(pywbem.CIMError,
                              self._mount,
                              '/no/such/dir',
                              partition,
                              fstype
                              )
            self.assertRaises(pywbem.CIMError,
                              self._umount,
                              '/no/such/dir',
                              partition,
                              fstype
                              )

    def test_mount_and_umount_wrong_partition(self):
        self.assertRaisesRegexp(AssertionError,
                                '^No associations with .*$',
                                self._mount,
                                self.mnt_dir,
                                '/no/such/partition',
                                'not_important'
                                 )

        self.assertRaises(pywbem.CIMError,
                          self._umount,
                          self.mnt_dir,
                          '/no/such/partition',
                          'not_important'
                          )

    def test_mount_and_umount_wrong_fstype(self):
        self.assertRaises(pywbem.CIMError,
                          self._mount,
                          self.mnt_dir,
                          self.mnt_partitions['ext4'],
                          'no-such-fs'
                          )

        self.assertRaises(pywbem.CIMError,
                          self._umount,
                          self.mnt_dir,
                          self.mnt_partitions['btrfs'],
                          'no-such-fs'
                          )

    def test_mount_own_simple_setting_ext4(self):
        setting_name = self._create_setting()
        setting = self.wbemconnection.GetInstance(setting_name)
        setting['AllowWrite'] = False
        setting['InterpretDevices'] = False
        otheropts = ['max_batch_time=30', 'nobarrier', 'journal_async_commit']
        setting['OtherOptions'] = otheropts
        self.wbemconnection.ModifyInstance(setting)

        fstype = 'ext4'
        partition = self.mnt_partitions[fstype]
        # mount
        try:
            self._mount(self.mnt_dir, partition, fstype, setting.path)
        except pywbem.CIMError as pe:
            self.fail(pe[1])
        out = self._grep_mount(partition)
        self._check_partition_is_mounted(partition)

        # check if settings are correctly set
        out = self._grep_mount(partition)
        # parse mount options in grep output (options are in parens):
        # /dev/vda7 on /mnt type ext4 (ro,nodev,...)
        opts = re.findall("\((.+),?\)", out)[0].split(',')

        # check if all expected options are in place
        self.assertIn('ro', opts)
        self.assertIn('nodev', opts)
        map(lambda o: self.assertIn(o, opts), otheropts)

        # unmount
        try:
            self._umount(self.mnt_dir, partition, fstype)
        except pywbem.CIMError as pe:
            self.fail(pe[1])
        out = self._grep_mount(partition)
        self._check_partition_is_unmounted(partition)

    # TODO more settings coverage for various filesystem types
    # more advanced settings tests

if __name__ == '__main__':
    unittest.main()
